package wusthelper.web.api.v2.module.undergrade;

import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import wusthelper.web.api.v2.TokenTool;
import wusthelper.web.api.v2.dto.response.CourseResponse;
import wusthelper.web.api.v2.dto.response.ScoreResponse;
import wusthelper.web.api.v2.dto.response.StudentInfoResponse;
import wusthelper.web.response.Response;
import wusthelper.web.service.campus.undergrad.*;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@RestController
@RequestMapping("/v2/jwc")
public class UndergradController {

    private final UndergradStudentInfoService studentInfoService;
    private final UndergradCourseTableService courseTableService;
    private final UndergradScoreService scoreService;
    private final UndergradCreditStatusService creditStatusService;
    private final UndergradTrainingPlanService trainingPlanService;

    public UndergradController(UndergradStudentInfoService studentInfoService,
                               UndergradCourseTableService courseTableService,
                               UndergradScoreService scoreService,
                               UndergradCreditStatusService creditStatusService,
                               UndergradTrainingPlanService trainingPlanService) {

        this.studentInfoService = studentInfoService;
        this.courseTableService = courseTableService;
        this.scoreService = scoreService;
        this.creditStatusService = creditStatusService;
        this.trainingPlanService = trainingPlanService;
    }

    @RequestMapping("/get-student-info")
    public Response<StudentInfoResponse> getStudentInfo(@RequestHeader("Token") String token) {
        var user = TokenTool.getStudentNumber(token);
        var studentInfo = studentInfoService.getStudentInfo(user);

        return Response.success(StudentInfoResponse.from(studentInfo));
    }

    @RequestMapping("/get-curriculum")
    public Response<List<CourseResponse>> getCourses(@RequestHeader("Token") String token,
                                                     @RequestParam(value = "schoolTerm") String term) {
        var user = TokenTool.getStudentNumber(token);
        var courses = courseTableService.getCourseTable(user, term);
        var courseResponsesList = new ArrayList<CourseResponse>(courses.size());
        for (var course : courses) {
            courseResponsesList.add(CourseResponse.from(course));
        }

        return Response.success(courseResponsesList);
    }

    @RequestMapping("/get-grade")
    public Response<List<ScoreResponse>> getScore(@RequestHeader("Token") String token) {
        var user = TokenTool.getStudentNumber(token);
        var scores = scoreService.getScore(user);
        var scoreResponseList = new ArrayList<ScoreResponse>(scores.size());

        // 课程
        Set<String> courseSet = new HashSet<>(scores.size());
        // 课程+学期
        Set<String> courseTermSet = new HashSet<>(scores.size());

        for (var score : scores) {
            var scoreResponse = ScoreResponse.from(score);

            // 重修判断 0--未重修  1--重修
            // 课程是否重复出现
            scoreResponse.setRebuildTag(0);
            String courseNumAndSchoolTerm = score.getCourseNumber() + score.getTerm();
            if (courseSet.contains(score.getCourseNumber())) {
                // 本次考试非补考 且本学期只有一次本课程号的课程，则表明本此考试为重修
                if (scoreResponse.getReExamTag() == 0 && !courseTermSet.contains(courseNumAndSchoolTerm)) {
                    scoreResponse.setRebuildTag(1);
                }
            }
            courseSet.add(score.getCourseNumber());
            courseTermSet.add(courseNumAndSchoolTerm);

            // 缓考不显示
            if (scoreResponse.getMissExamTag() != 2) {
                scoreResponseList.add(scoreResponse);
            }
        }

        return Response.success(scoreResponseList);
    }

    @RequestMapping("/get-credit")
    public Response<String> getCreditStatus(@RequestHeader("Token") String token) {
        var user = TokenTool.getStudentNumber(token);
        var page = creditStatusService.getCreditStatus(user);

        return Response.success(page);
    }

    @RequestMapping("/get-scheme")
    public Response<String> getTrainingPlan(@RequestHeader("Token") String token) {
        var user = TokenTool.getStudentNumber(token);
        var page = trainingPlanService.getTrainingPlan(user);

        return Response.success(page);
    }
}
