package wusthelper.web.service.campus;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.yitter.idgen.YitIdHelper;
import org.springframework.stereotype.Service;
import wusthelper.data.dao.mapper.StudentMapper;
import wusthelper.data.dao.mapper.UserBasicMapper;
import wusthelper.data.entity.Student;
import wusthelper.data.entity.UserBasic;
import wusthelper.web.data.entity.CookieType;
import wusthelper.web.service.campus.graduate.GraduateCookieService;
import wusthelper.web.service.campus.undergrad.UndergradCookieService;
import wusthelper.web.service.cookie.CookieManager;
import wusthelper.web.util.PasswordCodec;

/**
 * 通用的用户登录信息服务，包括登录和（已存储的）用户信息获取等，同时自动存入新用户基本（）信息
 */
@Service
public class GeneralUserLoginService {
    private final UserBasicMapper studentMapper;
    private final PasswordCodec passwordCodec;

    private final UndergradCookieService undergradCookieService;
    private final GraduateCookieService graduateCookieService;


    public GeneralUserLoginService(UserBasicMapper studentMapper,
                                   PasswordCodec passwordCodec,

                                   UndergradCookieService undergradCookieService,
                                   GraduateCookieService graduateCookieService) {

        this.studentMapper = studentMapper;
        this.passwordCodec = passwordCodec;

        this.undergradCookieService = undergradCookieService;
        this.graduateCookieService = graduateCookieService;
    }

    public enum UserType {Undergrad, Graduate}

    /**
     * 登录验证，同时自动存库（基本信息）
     *
     * @param username 用户名（即学号）
     * @param password 密码（明文原文）
     * @param userType 用户类型，现在有本科生和研究生
     * @return 用户信息
     */
    public UserBasic login(String username, String password, UserType userType) {
        var cookie = switch (userType) {
            case Undergrad -> undergradCookieService.getLoginCookie(username, password);
            case Graduate -> graduateCookieService.getLoginCookie(username, password);
        };

        var cookieType = switch (userType) {
            case Undergrad -> CookieType.Undergrad;
            case Graduate -> CookieType.Graduate;
        };

        // 保存cookie，并将用户基本信息存库
        CookieManager.store(username, cookie, cookieType);
        return this.saveOrUpdateUser(username, password, userType);
    }

    private UserBasic saveOrUpdateUser(String username, String password, UserType userType) {
        var query = new QueryWrapper<UserBasic>()
                .eq("stu_num", username)
                .eq("deleted", 0)
                .last("limit 1");

        var user = studentMapper.selectOne(query);
        if (user != null) {
            var encrypted = passwordCodec.encode(password);
            user.setOfficialPwd(encrypted);
            if (userType == UserType.Undergrad) {
                user.setLibPwd(encrypted);
            }
            studentMapper.update(user, query);
        } else {
            var uid = YitIdHelper.nextId();
            user = UserBasic.builder()
                    .uid(uid)
                    .stuNum(username)
                    .build();

            var encrypted = passwordCodec.encode(password);
            user.setOfficialPwd(encrypted);
            if (userType == UserType.Undergrad) {
                user.setLibPwd(encrypted);
            }

            studentMapper.insert(user);
        }

        return user;
    }
}
