package wusthelper.internal.graduate.services;

import cn.wustlinghang.mywust.core.request.service.auth.GraduateLogin;
import cn.wustlinghang.mywust.exception.ApiException;
import cn.wustlinghang.mywust.network.RequestClientOption;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.microprofile.config.inject.ConfigProperty;
import wusthelper.code.ServiceCode;
import wusthelper.internal.rpc.exception.RpcException;

import java.io.IOException;

@Slf4j
@ApplicationScoped
public class LoginService {

    private final RequestClientOption requestClientOption;
    private final GraduateLogin graduateLogin;
    private final int maxRetryTimes;

    public LoginService(RequestClientOption requestClientOption,
                        GraduateLogin graduateLogin,
                        @ConfigProperty(name = "graduate.max-retry-times", defaultValue = "4")
                        int maxRetryTimes) {

        this.requestClientOption = requestClientOption;
        this.graduateLogin = graduateLogin;

        this.maxRetryTimes = maxRetryTimes;
    }

    public String login(String username, String password) throws RpcException {
        try {
            return graduateLogin.getLoginCookie(username, password, maxRetryTimes, requestClientOption);
        } catch (ApiException e) {
            throw wrapApiException(e);
        } catch (IOException e) {
            throw new RpcException(ServiceCode.NetworkError);
        }
    }

    public boolean verify(String cookie) throws RpcException {
        try {
            graduateLogin.checkCookies(cookie, requestClientOption);
            return true;
        } catch (IOException e) {
            throw new RpcException(ServiceCode.NetworkError);
        } catch (ApiException e) {
            if (e.getCode() == ApiException.Code.NETWORK_EXCEPTION) {
                throw new RpcException(ServiceCode.NetworkError);
            }

            return false;
        }
    }

    private RpcException wrapApiException(ApiException e) {
        int errorCode = switch (e.getCode()) {
            case NETWORK_EXCEPTION -> ServiceCode.NetworkError;
            case GRADUATE_PASSWORD_WRONG -> ServiceCode.GraduatePasswordWrong;
            case GRADUATE_CAPTCHA_WRONG -> ServiceCode.GraduateCaptchaWrong;
            default -> {
                log.error("研究生：登录代理请求异常，异常未处理");
                log.error("异常：", e);
                yield ServiceCode.Unknown;
            }
        };

        return new RpcException(errorCode);
    }
}
